﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("DTLZ8", "Testfunction as defined as DTLZ7 in http://repository.ias.ac.in/81671/ [30.11.15]")]
  [StorableClass]
  public class DTLZ8 : DTLZ {


    [StorableConstructor]
    protected DTLZ8(bool deserializing) : base(deserializing) { }
    protected DTLZ8(DTLZ8 original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DTLZ8(this, cloner);
    }
    public DTLZ8() : base() { }

    public  double[] Evaluate2(RealVector r, int objectives) {
      if (r.Length < objectives) {
        throw new Exception("The dimensionality of the problem(ProblemSize) must be larger than or equal to ten times the dimensionality of the solution(SolutionSize) ");
      }
      double[] res = new double[objectives];

      //calculate f0...fM-1;
      double n = 10 * objectives;
      for (int i = 1; i <= objectives; i++) {
        double d = 0;
        int c = 0;
        for (int j = (int)Math.Floor((i - 1) * n / objectives); j < Math.Min((int)Math.Floor(i * n / objectives), r.Length); j++) {
          d += r[j];
          c++;
        }
        if (c != 0) {
          d *= 1.0 / c;
        } 
        res[i - 1] = d;
      }

      //evaluate constraints g0...gM-2
      for (int i = 0; i < objectives - 1; i++) {
        if (res[objectives - 1] + 4 * res[i] - 1 < 0) return MultiObjectiveTestFunction.IllegalValue(objectives, Maximization(objectives));
      }
      //evaluate gM-1
      double min = Double.PositiveInfinity;
      for (int i = 0; i < objectives - 1; i++) {
        for (int j = 0; j < i; j++) min = Math.Min(min, res[i] + res[j]);
      };
      if (2 * res[objectives - 1] + min - 1 < 0) return MultiObjectiveTestFunction.IllegalValue(objectives, Maximization(objectives));

      return res;
    }


    public override double[] Evaluate(RealVector r, int objectives) {
      if (r.Length < 10*objectives) throw new Exception("The dimensionality of the problem(ProblemSize) must be larger than or equal to ten times the dimensionality of the solution(SolutionSize) ");
      double n = r.Length;
      double M = objectives;
      double ratio = n / M;
      double[] res = new double[objectives];
      for(int j =0; j < objectives; j++) {
        double sum = 0;
        for(int i = (int) (j*ratio); i < (j+1)+ratio; i++) {
          sum+=r[i];
        }
        sum /= (int) ratio;
        res[j] = sum;
      }
      for(int j = 0; j < M - 1; j++) {
        if(res[objectives - 1]+4*res[j]-1<0) return MultiObjectiveTestFunction.IllegalValue(objectives, Maximization(objectives));
      }
      double min = Double.PositiveInfinity;
      for(int i =0; i < res.Length - 1; i++) {
        for(int j = 0;j< i; j++) {
          double d = res[i] + res[j];
          if (min < d) min = d;
        }
      }

      if (2*res[objectives-1]+min-1<0) return MultiObjectiveTestFunction.IllegalValue(objectives, Maximization(objectives));
      return res;
    }


  }
}
