﻿using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("SchafferN2", "Schaffer function N.2 for mulitobjective optimization from // https://en.wikipedia.org/wiki/Test_functions_for_optimization [30.11.2015]")]
  [StorableClass]
  public class SchafferN2 : MultiObjectiveTestFunction {

    public override double[,] Bounds(int objectives) {
      return new double[,] { { -5, 10 } };
    }

    public override bool[] Maximization(int objecitves) {
      return new bool[2];
    }

    public override int MinimumSolutionLength {
      get { return 1; }
    }

    public override int MaximumSolutionLength {
      get { return 1; }
    }

    public override int MinimumObjectives {
      get { return 2; }
    }

    public override int MaximumObjectives {
      get { return 2; }
    }

    public override double[] ReferencePoint(int objecitves) {
      return new double[] { 100, 100 };
    }


    public override IEnumerable<double[]> OptimalParetoFront(int objectives) {
      throw new NotImplementedException();
    }

    [StorableConstructor]
    protected SchafferN2(bool deserializing) : base(deserializing) { }
    protected SchafferN2(SchafferN2 original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new SchafferN2(this, cloner);
    }

    public SchafferN2() : base() { }





    public override double[] Evaluate(RealVector r, int objectives) {
      if (objectives != 2) throw new ArgumentException("The Schaffer N1 problem must always have 2 objectives");
      double x = r[0];

      //objective1
      double f0;
      if (x <= 1) f0 = -x;
      else if (x <= 3) f0 = x - 2;
      else if (x <= 4) f0 = 4 - x;
      else f0 = x - 4;

      //objective0
      double f1 = x - 5;
      f1 *= f1;

      return new double[] { f0, f1 };
    }
  }
}
